@@ -30,6 +30,12 @@ class _RayRunnerConfig(_RunnerConfig):
     address: str | None
 
 
+@dataclasses.dataclass(frozen=True)
+class _DynamicRayRunnerConfig(_RunnerConfig):
+    name = "dynamicray"
+    address: str | None
+
+
 def _get_runner_config_from_env() -> _RunnerConfig:
     """Retrieves the appropriate RunnerConfig from environment variables
 
@@ -46,6 +52,8 @@ def _get_runner_config_from_env() -> _RunnerConfig:
             return _PyRunnerConfig()
         elif runner.upper() == "DYNAMIC":
             return _DynamicRunnerConfig()
+        elif runner.upper() == "DYNAMICRAY":
+            return _DynamicRayRunnerConfig(address=os.getenv("DAFT_RAY_ADDRESS"))
         raise ValueError(f"Unsupported DAFT_RUNNER variable: {os.environ['DAFT_RUNNER']}")
     return _PyRunnerConfig()
 
@@ -82,6 +90,13 @@ class DaftContext:
 
             logger.info("Using DynamicRunner")
             _RUNNER = DynamicRunner()
+
+        elif self.runner_config.name == "dynamicray":
+            from daft.runners.ray_runner import DynamicRayRunner
+
+            logger.info("Using DynamicRayRunner")
+            assert isinstance(self.runner_config, _DynamicRayRunnerConfig)
+            _RUNNER = DynamicRayRunner(address=self.runner_config.address)
         else:
             raise NotImplementedError(f"Runner config implemented: {self.runner_config.name}")
 
@@ -104,7 +119,7 @@ def get_context() -> DaftContext:
 
 
 def set_runner_ray(address: str | None = None) -> DaftContext:
-    """Sets the runner for executing Daft dataframes to the RayRunner
+    """Set the runner for executing Daft dataframes to a Ray cluster
 
     Alternatively, users can set this behavior via environment variables:
 
@@ -128,8 +143,21 @@ def set_runner_ray(address: str | None = None) -> DaftContext:
     return _DaftContext
 
 
+def set_runner_dynamic_ray(address: str | None = None) -> DaftContext:
+    """[Experimental] Sets the runner for executing Daft dataframes to the DynamicRayRunner."""
+    global _DaftContext
+    if _DaftContext.disallow_set_runner:
+        raise RuntimeError("Cannot set runner more than once")
+    _DaftContext = dataclasses.replace(
+        _DaftContext,
+        runner_config=_DynamicRayRunnerConfig(address=address),
+        disallow_set_runner=True,
+    )
+    return _DaftContext
+
+
 def set_runner_py() -> DaftContext:
-    """Sets the runner for executing Daft dataframes to the PyRunner. This is the default behavior.
+    """Set the runner for executing Daft dataframes to your local Python interpreter - this is the default behavior.
 
     Alternatively, users can set this behavior via an environment variable: DAFT_RUNNER=py
 
@@ -148,6 +176,7 @@ def set_runner_py() -> DaftContext:
 
 
 def set_runner_dynamic() -> DaftContext:
+    """[Experimental] Sets the runner for executing Daft dataframes to the DynamicRunner."""
     global _DaftContext
     if _DaftContext.disallow_set_runner:
         raise RuntimeError("Cannot set runner more than once")
