@@ -7,6 +7,7 @@ from typing import TYPE_CHECKING, Any, Callable, Iterable, TypeVar, Union
 import pandas
 import pyarrow as pa
 
+from daft.api_annotations import DataframePublicAPI
 from daft.context import get_context
 from daft.dataframe.preview import DataFramePreview
 from daft.dataframe.schema import DataFrameSchema
@@ -122,6 +123,7 @@ class DataFrame:
         """
         return self.__plan
 
+    @DataframePublicAPI
     def explain(self, show_optimized: bool = False) -> None:
         """Prints the LogicalPlan that will be executed to produce this DataFrame.
         Defaults to showing the unoptimized plan. Use `show_optimized` to show the optimized one.
@@ -144,6 +146,7 @@ class DataFrame:
     def num_partitions(self) -> int:
         return self.__plan.num_partitions()
 
+    @DataframePublicAPI
     def schema(self) -> DataFrameSchema:
         """Returns the DataFrameSchema of the DataFrame, which provides information about each column
 
@@ -170,17 +173,18 @@ class DataFrame:
         """
         return [expr.to_column_expression() for expr in self.__plan.schema()]
 
+    @DataframePublicAPI
     def show(self, n: int | None = None) -> DataFrameDisplay:
-        """Executes and displays the executed dataframe as a table
+        """Executes enough of the DataFrame in order to display the first ``n`` rows
+
+        .. NOTE::
+            This call is **blocking** and will execute the DataFrame when called
 
         Args:
             n: number of rows to show. Defaults to None which indicates showing the entire Dataframe.
 
         Returns:
             DataFrameDisplay: object that has a rich tabular display
-
-            .. NOTE::
-                This call is **blocking** and will execute the DataFrame when called
         """
         df = self
         if n is not None:
@@ -200,10 +204,12 @@ class DataFrame:
 
         return DataFrameDisplay(preview, df.schema())
 
+    @DataframePublicAPI
     def __repr__(self) -> str:
         display = DataFrameDisplay(self._preview, self.schema())
         return display.__repr__()
 
+    @DataframePublicAPI
     def _repr_html_(self) -> str:
         display = DataFrameDisplay(self._preview, self.schema())
         return display._repr_html_()
@@ -213,6 +219,7 @@ class DataFrame:
     ###
 
     @classmethod
+    @DataframePublicAPI
     def from_pylist(cls, data: list[dict[str, Any]]) -> DataFrame:
         """Creates a DataFrame from a list of dictionaries
 
@@ -230,8 +237,9 @@ class DataFrame:
         return cls.from_pydict(data={header: [row[header] for row in data] for header in data[0]})
 
     @classmethod
+    @DataframePublicAPI
     def from_pydict(cls, data: dict[str, list | pa.Array]) -> DataFrame:
-        """Creates a DataFrame from a Python Dictionary.
+        """Creates a DataFrame from a Python dictionary
 
         Example:
             >>> df = DataFrame.from_pydict({"foo": [1, 2]})
@@ -287,11 +295,13 @@ class DataFrame:
         return cls(plan)
 
     @classmethod
+    @DataframePublicAPI
     def from_json(cls, *args, **kwargs) -> DataFrame:
         warnings.warn(f"DataFrame.from_json will be deprecated in 0.1.0 in favor of DataFrame.read_json")
         return cls.read_json(*args, **kwargs)
 
     @classmethod
+    @DataframePublicAPI
     def read_json(
         cls,
         path: str,
@@ -333,11 +343,13 @@ class DataFrame:
         return cls(plan)
 
     @classmethod
+    @DataframePublicAPI
     def from_csv(cls, *args, **kwargs) -> DataFrame:
         warnings.warn(f"DataFrame.from_csv will be deprecated in 0.1.0 in favor of DataFrame.read_csv")
         return cls.read_csv(*args, **kwargs)
 
     @classmethod
+    @DataframePublicAPI
     def read_csv(
         cls,
         path: str,
@@ -394,11 +406,13 @@ class DataFrame:
         return cls(plan)
 
     @classmethod
+    @DataframePublicAPI
     def from_parquet(cls, *args, **kwargs) -> DataFrame:
         warnings.warn(f"DataFrame.from_parquet will be deprecated in 0.1.0 in favor of DataFrame.read_parquet")
         return cls.read_parquet(*args, **kwargs)
 
     @classmethod
+    @DataframePublicAPI
     def read_parquet(cls, path: str) -> DataFrame:
         """Creates a DataFrame from Parquet file(s)
 
@@ -437,8 +451,9 @@ class DataFrame:
         return cls(plan)
 
     @classmethod
+    @DataframePublicAPI
     def from_files(cls, path: str) -> DataFrame:
-        """Creates a DataFrame from files in storage, where each file is one row of the DataFrame
+        """Creates a DataFrame of file paths and other metadata from a glob path
 
         Example:
             >>> df = DataFrame.from_files("/path/to/files/*.jpeg")
@@ -458,12 +473,19 @@ class DataFrame:
     # Write methods
     ###
 
+    @DataframePublicAPI
     def write_parquet(
         self, root_dir: str, compression: str = "snappy", partition_cols: list[ColumnInputType] | None = None
     ) -> DataFrame:
-        """Writes the DataFrame to parquet files using a `root_dir` and randomly generated UUIDs as the filepath and returns the filepaths.
+        """Writes the DataFrame as parquet files, returning a new DataFrame with paths to the files that were written
+
+        Files will be written to ``<root_dir>/*`` with randomly generated UUIDs as the file names.
+
         Currently generates a parquet file per partition unless `partition_cols` are used, then the number of files can equal the number of partitions times the number of values of partition col.
 
+        .. NOTE::
+            This call is **blocking** and will execute the DataFrame when called
+
         Args:
             root_dir (str): root file path to write parquet files to.
             compression (str, optional): compression algorithm. Defaults to "snappy".
@@ -497,10 +519,17 @@ class DataFrame:
         assert write_df._result is not None
         return DataFrame(write_df._plan)
 
+    @DataframePublicAPI
     def write_csv(self, root_dir: str, partition_cols: list[ColumnInputType] | None = None) -> DataFrame:
-        """Writes the DataFrame to CSV files using a `root_dir` and randomly generated UUIDs as the filepath and returns the filepaths.
+        """Writes the DataFrame as CSV files, returning a new DataFrame with paths to the files that were written
+
+        Files will be written to ``<root_dir>/*`` with randomly generated UUIDs as the file names.
+
         Currently generates a csv file per partition unless `partition_cols` are used, then the number of files can equal the number of partitions times the number of values of partition col.
 
+        .. NOTE::
+            This call is **blocking** and will execute the DataFrame when called
+
         Args:
             root_dir (str): root file path to write parquet files to.
             compression (str, optional): compression algorithm. Defaults to "snappy".
@@ -508,9 +537,6 @@ class DataFrame:
 
         Returns:
             DataFrame: The filenames that were written out as strings.
-
-            .. NOTE::
-                This call is **blocking** and will execute the DataFrame when called
         """
         cols: ExpressionList | None = None
         if partition_cols is not None:
@@ -542,6 +568,7 @@ class DataFrame:
         return ExpressionList(expressions)
 
     def __getitem__(self, item: slice | int | str | Iterable[str | int]) -> ColumnExpression | DataFrame:
+        """Gets a column from the DataFrame as an Expression (``df["mycol"]``)"""
         result: ColumnExpression | None
 
         if isinstance(item, int):
@@ -582,8 +609,9 @@ class DataFrame:
         else:
             raise ValueError(f"unknown indexing type: {type(item)}")
 
+    @DataframePublicAPI
     def select(self, *columns: ColumnInputType) -> DataFrame:
-        """Creates a new DataFrame that `selects` that columns that are passed in from the current DataFrame.
+        """Creates a new DataFrame from the provided expressions, similar to a SQL ``SELECT``
 
         Example:
 
@@ -609,8 +637,9 @@ class DataFrame:
         projection = logical_plan.Projection(self._plan, self.__column_input_to_expression(columns))
         return DataFrame(projection)
 
+    @DataframePublicAPI
     def distinct(self) -> DataFrame:
-        """Computes unique rows, dropping duplicates.
+        """Computes unique rows, dropping duplicates
 
         Example:
             >>> unique_df = df.distinct()
@@ -630,8 +659,10 @@ class DataFrame:
             plan = logical_plan.LocalDistinct(plan, all_exprs)
         return DataFrame(plan)
 
+    @DataframePublicAPI
     def exclude(self, *names: str) -> DataFrame:
-        """Drops columns from the current DataFrame by name.
+        """Drops columns from the current DataFrame by name
+
         This is equivalent of performing a select with all the columns but the ones excluded.
 
         Example:
@@ -647,9 +678,9 @@ class DataFrame:
         el = ExpressionList([e for e in self._plan.schema() if e.name() not in names_to_skip])
         return DataFrame(logical_plan.Projection(self._plan, el))
 
+    @DataframePublicAPI
     def where(self, predicate: Expression) -> DataFrame:
-        """Filters rows via a predicate expression.
-        similar to SQL style `where`.
+        """Filters rows via a predicate expression, similar to SQL ``WHERE``.
 
         Example:
             >>> filtered_df = df.where((col('x') < 10) & (col('y') == 10))
@@ -663,9 +694,10 @@ class DataFrame:
         plan = logical_plan.Filter(self._plan, ExpressionList([predicate]))
         return DataFrame(plan)
 
+    @DataframePublicAPI
     def with_column(self, column_name: str, expr: Expression) -> DataFrame:
-        """Adds a column to the current DataFrame with an Expression.
-        This is equivalent to performing a `select` with all the current columns and the new one.
+        """Adds a column to the current DataFrame with an Expression, equivalent to a ``select``
+        with all current columns and the new one
 
         Example:
             >>> new_df = df.with_column('x+1', col('x') + 1)
@@ -683,8 +715,9 @@ class DataFrame:
         )
         return DataFrame(projection)
 
+    @DataframePublicAPI
     def sort(self, by: ColumnInputType | list[ColumnInputType], desc: bool | list[bool] = False) -> DataFrame:
-        """Sorts DataFrame globally according to column.
+        """Sorts DataFrame globally
 
         Example:
             >>> sorted_df = df.sort(col('x') + col('y'))
@@ -708,9 +741,9 @@ class DataFrame:
         sort = logical_plan.Sort(self._plan, self.__column_input_to_expression(by), descending=desc)
         return DataFrame(sort)
 
+    @DataframePublicAPI
     def limit(self, num: int) -> DataFrame:
-        """Limits the rows returned by the DataFrame via a `head` operation.
-        This is similar to how `limit` works in SQL.
+        """Limits the rows in the DataFrame to the first ``N`` rows, similar to a SQL ``LIMIT``
 
         Example:
             >>> df_limited = df.limit(10) # returns 10 rows
@@ -725,8 +758,9 @@ class DataFrame:
         global_limit = logical_plan.GlobalLimit(local_limit, num=num)
         return DataFrame(global_limit)
 
+    @DataframePublicAPI
     def repartition(self, num: int, *partition_by: ColumnInputType) -> DataFrame:
-        """Repartitions DataFrame to `num` partitions.
+        """Repartitions DataFrame to ``num`` partitions
 
         If columns are passed in, then DataFrame will be repartitioned by those, otherwise
         random repartitioning will occur.
@@ -753,6 +787,7 @@ class DataFrame:
         repartition_op = logical_plan.Repartition(self._plan, num_partitions=num, partition_by=exprs, scheme=scheme)
         return DataFrame(repartition_op)
 
+    @DataframePublicAPI
     def join(
         self,
         other: DataFrame,
@@ -761,8 +796,7 @@ class DataFrame:
         right_on: list[ColumnInputType] | ColumnInputType | None = None,
         how: str = "inner",
     ) -> DataFrame:
-        """Joins left (self) DataFrame on the right on a set of keys.
-        Key names can be the same or different for left and right DataFrame.
+        """Column-wise join of the current DataFrame with an ``other`` DataFrame, similar to a SQL ``JOIN``
 
         .. NOTE::
             Although self joins are supported, we currently duplicate the logical plan for the right side
@@ -799,9 +833,10 @@ class DataFrame:
         )
         return DataFrame(join_op)
 
+    @DataframePublicAPI
     def explode(self, *columns: ColumnInputType) -> DataFrame:
         """Explodes a List column, where every element in each row's List becomes its own row, and all
-        other columns in the DataFrame are duplicated across rows.
+        other columns in the DataFrame are duplicated across rows
 
         If multiple columns are specified, each row must contain the same number of
         items in each specified column.
@@ -967,8 +1002,9 @@ class DataFrame:
 
         return DataFrame(final_op)
 
+    @DataframePublicAPI
     def sum(self, *cols: ColumnInputType) -> DataFrame:
-        """Performs a global sum on the DataFrame on a sequence of columns.
+        """Performs a global sum on the DataFrame
 
         Args:
             *cols (Union[str, Expression]): columns to sum
@@ -978,8 +1014,9 @@ class DataFrame:
         assert len(cols) > 0, "no columns were passed in"
         return self._agg([(c, "sum") for c in cols])
 
+    @DataframePublicAPI
     def mean(self, *cols: ColumnInputType) -> DataFrame:
-        """Performs a global mean on the DataFrame on a sequence of columns.
+        """Performs a global mean on the DataFrame
 
         Args:
             *cols (Union[str, Expression]): columns to mean
@@ -989,8 +1026,9 @@ class DataFrame:
         assert len(cols) > 0, "no columns were passed in"
         return self._agg([(c, "mean") for c in cols])
 
+    @DataframePublicAPI
     def min(self, *cols: ColumnInputType) -> DataFrame:
-        """Performs a global min on the DataFrame on a sequence of columns.
+        """Performs a global min on the DataFrame
 
         Args:
             *cols (Union[str, Expression]): columns to min
@@ -1000,8 +1038,9 @@ class DataFrame:
         assert len(cols) > 0, "no columns were passed in"
         return self._agg([(c, "min") for c in cols])
 
+    @DataframePublicAPI
     def max(self, *cols: ColumnInputType) -> DataFrame:
-        """Performs a global max on the DataFrame on a sequence of columns.
+        """Performs a global max on the DataFrame
 
         Args:
             *cols (Union[str, Expression]): columns to max
@@ -1011,8 +1050,9 @@ class DataFrame:
         assert len(cols) > 0, "no columns were passed in"
         return self._agg([(c, "max") for c in cols])
 
+    @DataframePublicAPI
     def count(self, *cols: ColumnInputType) -> DataFrame:
-        """Performs a global count on the DataFrame on a sequence of columns.
+        """Performs a global count on the DataFrame
 
         Args:
             *cols (Union[str, Expression]): columns to count
@@ -1022,18 +1062,19 @@ class DataFrame:
         assert len(cols) > 0, "no columns were passed in"
         return self._agg([(c, "count") for c in cols])
 
+    @DataframePublicAPI
     def agg(self, to_agg: list[tuple[ColumnInputType, str]]) -> DataFrame:
         """Perform aggregations on this DataFrame. Allows for mixed aggregations for multiple columns
         Will return a single row that aggregated the entire DataFrame.
 
         Example:
-        >>> df = df.agg([
-        >>>     ('x', 'sum'),
-        >>>     ('x', 'mean'),
-        >>>     ('y', 'min'),
-        >>>     ('y', 'max'),
-        >>>     (col('x') + col('y'), 'max'),
-        >>> ])
+            >>> df = df.agg([
+            >>>     ('x', 'sum'),
+            >>>     ('x', 'mean'),
+            >>>     ('y', 'min'),
+            >>>     ('y', 'max'),
+            >>>     (col('x') + col('y'), 'max'),
+            >>> ])
 
         Args:
             to_agg (List[Tuple[ColumnInputType, str]]): list of (column, agg_type)
@@ -1043,8 +1084,9 @@ class DataFrame:
         """
         return self._agg(to_agg, group_by=None)
 
+    @DataframePublicAPI
     def groupby(self, *group_by: ColumnInputType) -> GroupedDataFrame:
-        """Performs a GroupBy on the DataFrame for Aggregation.
+        """Performs a GroupBy on the DataFrame for aggregation
 
         Args:
             *group_by (Union[str, Expression]): columns to group by
@@ -1063,8 +1105,12 @@ class DataFrame:
             assert result is not None
             result.wait()
 
+    @DataframePublicAPI
     def collect(self, num_preview_rows: int | None = 10) -> DataFrame:
-        """Computes LogicalPlan to materialize DataFrame. This is a blocking operation.
+        """Executes the entire DataFrame and materializes the results
+
+        .. NOTE::
+            This call is **blocking** and will execute the DataFrame when called
 
         Args:
             num_preview_rows: Number of rows to preview. Defaults to 10
@@ -1097,6 +1143,7 @@ class DataFrame:
 
         return self
 
+    @DataframePublicAPI
     def to_pandas(self) -> pandas.DataFrame:
         """Converts the current DataFrame to a pandas DataFrame.
         If results have not computed yet, collect will be called.
@@ -1114,6 +1161,7 @@ class DataFrame:
         pd_df = result.to_pandas(schema=self._plan.schema())
         return pd_df
 
+    @DataframePublicAPI
     def to_pydict(self) -> pandas.DataFrame:
         """Converts the current DataFrame to a python dict.
         If results have not computed yet, collect will be called.
@@ -1129,6 +1177,7 @@ class DataFrame:
         assert result is not None
         return result.to_pydict()
 
+    @DataframePublicAPI
     def to_ray_dataset(self) -> RayDataset:
         """Converts the current DataFrame to a Ray Dataset which is useful for running distributed ML model training in Ray
 
@@ -1159,6 +1208,7 @@ class GroupedDataFrame:
                 raise ExpressionTypeError(f"Cannot groupby on null type expression: {e}")
 
     def __getitem__(self, item: slice | int | str | Iterable[str | int]) -> ColumnExpression | DataFrame:
+        """Gets a column from the DataFrame as an Expression"""
         return self.df.__getitem__(item)
 
     def sum(self, *cols: ColumnInputType) -> DataFrame:
@@ -1211,12 +1261,12 @@ class GroupedDataFrame:
         """Perform aggregations on this GroupedDataFrame. Allows for mixed aggregations.
 
         Example:
-        >>> df = df.groupby('x').agg([
-        >>>     ('x', 'sum'),
-        >>>     ('x', 'mean'),
-        >>>     ('y', 'min'),
-        >>>     ('y', 'max'),
-        >>> ])
+            >>> df = df.groupby('x').agg([
+            >>>     ('x', 'sum'),
+            >>>     ('x', 'mean'),
+            >>>     ('y', 'min'),
+            >>>     ('y', 'max'),
+            >>> ])
 
         Args:
             to_agg (List[Tuple[ColumnInputType, str]]): list of (column, agg_type)
