@@ -5,11 +5,6 @@ from bisect import bisect_right
 from itertools import accumulate
 from typing import Callable, ClassVar, TypeVar
 
-from fsspec.implementations.cached import SimpleCacheFileSystem
-from fsspec.implementations.http import HTTPFileSystem
-from pyarrow import csv, json, parquet
-from pyarrow.fs import FSSpecHandler, PyFileSystem
-
 from daft.datasources import (
     CSVSourceInfo,
     JSONSourceInfo,
@@ -17,7 +12,6 @@ from daft.datasources import (
     StorageType,
 )
 from daft.expressions import ColID
-from daft.filesystem import get_filesystem_from_path
 from daft.logical.logical_plan import (
     Coalesce,
     FileWrite,
@@ -33,13 +27,20 @@ from daft.logical.logical_plan import (
     PartitionScheme,
     Projection,
     Repartition,
-    Scan,
     Sort,
+    TabularFilesScan,
 )
 from daft.logical.schema import ExpressionList
 from daft.resource_request import ResourceRequest
 from daft.runners.blocks import DataBlock
-from daft.runners.partitioning import PartitionSet, PyListTile, vPartition
+from daft.runners.partitioning import (
+    PartitionSet,
+    PyListTile,
+    vPartition,
+    vPartitionParseCSVOptions,
+    vPartitionReadOptions,
+    vPartitionSchemaInferenceOptions,
+)
 from daft.runners.shuffle_ops import (
     CoalesceOp,
     RepartitionHashOp,
@@ -73,8 +74,8 @@ class LogicalPartitionOpRunner:
         return output
 
     def run_single_node(self, inputs: dict[int, vPartition], node: LogicalPlan, partition_id: int) -> vPartition:
-        if isinstance(node, Scan):
-            return self._handle_scan(inputs, node, partition_id=partition_id)
+        if isinstance(node, TabularFilesScan):
+            return self._handle_tabular_files_scan(inputs, node, partition_id=partition_id)
         elif isinstance(node, Projection):
             return self._handle_projection(inputs, node, partition_id=partition_id)
         elif isinstance(node, Filter):
@@ -94,49 +95,71 @@ class LogicalPartitionOpRunner:
         else:
             raise NotImplementedError(f"{type(node)} not implemented")
 
-    def _handle_scan(self, inputs: dict[int, vPartition], scan: Scan, partition_id: int) -> vPartition:
+    def _handle_tabular_files_scan(
+        self, inputs: dict[int, vPartition], scan: TabularFilesScan, partition_id: int
+    ) -> vPartition:
+        child_id = scan._children()[0].id()
+        prev_partition = inputs[child_id]
+        data = prev_partition.to_pydict()
+        assert (
+            scan._filepaths_column_name in data
+        ), f"TabularFilesScan should be ran on vPartitions with '{scan._filepaths_column_name}' column"
+        # HACK: Ignore type here because this is a PyArrow array
+        filepaths = data[scan._filepaths_column_name].to_pylist()  # type: ignore
+
+        # Common options for reading vPartition
         schema = scan._schema
+        schema_options = vPartitionSchemaInferenceOptions(schema=schema)
+        read_options = vPartitionReadOptions(
+            num_rows=None,  # read all rows
+            column_names=scan._column_names,  # read only specified columns
+        )
+
         if scan._source_info.scan_type() == StorageType.CSV:
             assert isinstance(scan._source_info, CSVSourceInfo)
-            path = scan._source_info.filepaths[partition_id]
-            fs = get_filesystem_from_path(path)
-            table = csv.read_csv(
-                fs.open(path, compression="infer"),
-                parse_options=csv.ParseOptions(
-                    delimiter=scan._source_info.delimiter,
-                ),
-                read_options=csv.ReadOptions(
-                    column_names=[expr.name() for expr in schema],
-                    skip_rows_after_names=1 if scan._source_info.has_headers else 0,
-                ),
-                convert_options=csv.ConvertOptions(include_columns=scan._column_names),
+            return vPartition.merge_partitions(
+                [
+                    vPartition.from_csv(
+                        path=fp,
+                        partition_id=partition_id,
+                        csv_options=vPartitionParseCSVOptions(
+                            delimiter=scan._source_info.delimiter,
+                            has_headers=scan._source_info.has_headers,
+                            skip_rows_before_header=0,
+                            skip_rows_after_header=0,
+                        ),
+                        schema_options=schema_options,
+                        read_options=read_options,
+                    )
+                    for fp in filepaths
+                ]
             )
-            column_ids = [col.get_id() for col in scan.schema().to_column_expressions()]
-            vpart = vPartition.from_arrow_table(table, column_ids=column_ids, partition_id=partition_id)
-            return vpart
         elif scan._source_info.scan_type() == StorageType.JSON:
             assert isinstance(scan._source_info, JSONSourceInfo)
-            path = scan._source_info.filepaths[partition_id]
-            fs = get_filesystem_from_path(path)
-            table = json.read_json(fs.open(path, compression="infer")).select([col.name() for col in scan.schema()])
-            column_ids = [col.get_id() for col in scan.schema().to_column_expressions()]
-            vpart = vPartition.from_arrow_table(table, column_ids=column_ids, partition_id=partition_id)
-            return vpart
+            return vPartition.merge_partitions(
+                [
+                    vPartition.from_json(
+                        path=fp,
+                        partition_id=partition_id,
+                        schema_options=schema_options,
+                        read_options=read_options,
+                    )
+                    for fp in filepaths
+                ]
+            )
         elif scan._source_info.scan_type() == StorageType.PARQUET:
             assert isinstance(scan._source_info, ParquetSourceInfo)
-
-            # PyArrow lacks a HTTP filesystem implementation, so we use FSSpec here to handle that case
-            pyarrow_fs = None
-            fsspec_fs = get_filesystem_from_path(scan._source_info.filepaths[partition_id])
-            if isinstance(fsspec_fs, HTTPFileSystem):
-                pyarrow_fs = PyFileSystem(FSSpecHandler(SimpleCacheFileSystem(fs=fsspec_fs)))
-
-            table = parquet.read_table(
-                scan._source_info.filepaths[partition_id], columns=scan._column_names, filesystem=pyarrow_fs
+            return vPartition.merge_partitions(
+                [
+                    vPartition.from_parquet(
+                        path=fp,
+                        partition_id=partition_id,
+                        schema_options=schema_options,
+                        read_options=read_options,
+                    )
+                    for fp in filepaths
+                ]
             )
-            column_ids = [col.get_id() for col in scan.schema().to_column_expressions()]
-            vpart = vPartition.from_arrow_table(table, column_ids=column_ids, partition_id=partition_id)
-            return vpart
         else:
             raise NotImplementedError(f"PyRunner has not implemented scan: {scan._source_info.scan_type()}")
 
@@ -311,7 +334,7 @@ class LogicalGlobalOpRunner:
         elif repartition._scheme == PartitionScheme.HASH:
             repartitioner = self._get_shuffle_op_klass(RepartitionHashOp)(
                 expr_eval_resource_request=repartition.resource_request(),
-                map_args={"exprs": repartition._partition_by.exprs},
+                map_args={"exprs": repartition._partition_by},
             )
         else:
             raise NotImplementedError()
