@@ -1,13 +1,22 @@
 from __future__ import annotations
 
 from dataclasses import dataclass
-from typing import TYPE_CHECKING, Any, Callable, ClassVar
+from datetime import datetime
+from typing import TYPE_CHECKING, Any, Callable, ClassVar, Iterator
 
 import pyarrow as pa
 import ray
 from loguru import logger
 
+from daft.execution import physical_plan_factory
 from daft.execution.execution_plan import ExecutionPlan
+from daft.execution.execution_step import (
+    Instruction,
+    MaterializationRequest,
+    MaterializationRequestBase,
+    MaterializationRequestMulti,
+    MaterializationResult,
+)
 from daft.execution.logical_op_runners import (
     LogicalGlobalOpRunner,
     LogicalPartitionOpRunner,
@@ -32,6 +41,7 @@ from daft.runners.blocks import ArrowDataBlock, zip_blocks_as_py
 from daft.runners.partitioning import (
     PartID,
     PartitionCacheEntry,
+    PartitionMetadata,
     PartitionSet,
     PartitionSetFactory,
     vPartition,
@@ -332,7 +342,7 @@ class RayRunner(Runner):
         exec_plan = ExecutionPlan.plan_from_logical(plan)
         result_partition_set: PartitionSet
         partition_intermediate_results: dict[int, PartitionSet] = {}
-        with profiler("profile.json"):
+        with profiler("profile_RayRunner.run_{datetime.now().isoformat()}.json"):
             for exec_op in exec_plan.execution_ops:
 
                 data_deps = exec_op.data_deps
@@ -357,3 +367,164 @@ class RayRunner(Runner):
             final_result = partition_intermediate_results[last.id()]
             pset_entry = self._part_set_cache.put_partition_set(final_result)
             return pset_entry
+
+
+@ray.remote
+def build_partitions(instruction_stack: list[Instruction], *inputs: vPartition) -> vPartition | list[vPartition]:
+    partitions = list(inputs)
+    for instruction in instruction_stack:
+        partitions = instruction.run(partitions)
+
+    return partitions if len(partitions) > 1 else partitions[0]
+
+
+@ray.remote
+def get_meta(partition: vPartition) -> PartitionMetadata:
+    return partition.metadata()
+
+
+class DynamicRayRunner(RayRunner):
+    def run(self, plan: logical_plan.LogicalPlan) -> PartitionCacheEntry:
+
+        plan = self.optimize(plan)
+
+        phys_plan: Iterator[
+            None | MaterializationRequestBase[vPartition]
+        ] = physical_plan_factory.get_materializing_physical_plan(plan)
+
+        # Note: For autoscaling clusters, we will probably want to query this dynamically.
+        # Keep in mind this call takes about 0.3ms.
+        cores = int(ray.cluster_resources()["CPU"])
+        constructions_to_dispatch = []
+        inflight_constructions: dict[str, MaterializationRequestBase[ray.ObjectRef]] = dict()
+        inflight_ref_to_construction: dict[ray.ObjectRef, str] = dict()
+
+        start = datetime.now()
+        result_pset = RayPartitionSet({})
+        with profiler("profile_DynamicRayRunner.run_{datetime.now().isoformat()}.json"):
+
+            try:
+
+                while True:
+                    # Dispatch tasks while cores are available.
+                    cores_available = cores - len(inflight_constructions)
+                    for i in range(cores_available):
+
+                        next_step = next(phys_plan)
+
+                        while next_step is not None and len(next_step.instructions) == 0:
+                            assert isinstance(next_step, MaterializationRequest)
+                            [partition] = next_step.inputs
+                            next_step.result = RayMaterializationResult(partition)
+                            next_step = next(phys_plan)
+
+                        if next_step is None:
+                            break
+
+                        constructions_to_dispatch.append(next_step)
+
+                    dispatch = datetime.now()
+                    logger.debug(
+                        f"{(dispatch - start).total_seconds()}s: DynamicRayRunner dispatched batch of {len(constructions_to_dispatch)} tasks."
+                    )
+                    for construction in constructions_to_dispatch:
+                        results = self._build_partitions(construction)
+                        logger.debug(f"{construction} -> {results}")
+                        inflight_constructions[construction.id()] = construction
+                        for result in results:
+                            inflight_ref_to_construction[result] = construction.id()
+
+                    constructions_to_dispatch.clear()
+
+                    # All tasks dispatched. Await a result
+                    # while inflight_ref_to_construction:
+                    dispatch = datetime.now()
+                    [ready], _ = ray.wait(list(inflight_ref_to_construction.keys()), fetch_local=False)
+                    cons_id = inflight_ref_to_construction[ready]
+                    logger.debug(f"+{(datetime.now() - dispatch).total_seconds()}s to await a result from {cons_id}")
+
+                    # Flush the entire task associated with the result
+
+                    cons = inflight_constructions[cons_id]
+                    if isinstance(cons, MaterializationRequest):
+                        del inflight_ref_to_construction[ready]
+                    elif isinstance(cons, MaterializationRequestMulti):
+
+                        assert cons.results is not None
+                        for result in cons.results:
+                            del inflight_ref_to_construction[result.partition()]
+
+                    del inflight_constructions[cons_id]
+
+            except StopIteration as e:
+                for i, partition in enumerate(result_partitions := e.value):
+                    result_pset.set_partition(i, partition)
+
+            for construction in constructions_to_dispatch:
+                results = self._build_partitions(construction)
+                inflight_constructions[construction.id()] = construction
+                for result in results:
+                    inflight_ref_to_construction[result] = construction.id()
+
+            constructions_to_dispatch.clear()
+
+            while inflight_ref_to_construction:
+                dispatch = datetime.now()
+                [ready], _ = ray.wait(list(inflight_ref_to_construction.keys()), fetch_local=False)
+                cons_id = inflight_ref_to_construction[ready]
+                logger.debug(f"+{(datetime.now() - dispatch).total_seconds()}s to await a result of {cons_id}")
+
+                # Flush the entire task associated with the result
+                cons = inflight_constructions[cons_id]
+                if isinstance(cons, MaterializationRequest):
+                    del inflight_ref_to_construction[ready]
+                elif isinstance(cons, MaterializationRequestMulti):
+
+                    assert cons.results is not None
+                    for result in cons.results:
+                        del inflight_ref_to_construction[result.partition()]
+
+                del inflight_constructions[cons_id]
+
+        pset_entry = self._part_set_cache.put_partition_set(result_pset)
+        return pset_entry
+
+    def _build_partitions(self, partspec: MaterializationRequestBase[ray.ObjectRef]) -> list[ray.ObjectRef]:
+        ray_options: dict[str, Any] = {
+            "num_returns": partspec.num_results,
+        }
+        if len(partspec.inputs) > 1:
+            ray_options["scheduling_strategy"] = "SPREAD"
+
+        construct_remote = build_partitions.options(**ray_options)
+        partitions = construct_remote.remote(partspec.instructions, *partspec.inputs)
+        # Handle ray bug that ignores list interpretation when num_returns=1
+        if partspec.num_results == 1:
+            partitions = [partitions]
+
+        if isinstance(partspec, MaterializationRequestMulti):
+            partspec.results = [RayMaterializationResult(partition) for partition in partitions]
+        elif isinstance(partspec, MaterializationRequest):
+            [partition] = partitions
+            partspec.result = RayMaterializationResult(partition)
+        else:
+            raise TypeError(f"Could not type match input {partspec}")
+
+        return partitions
+
+
+@dataclass(frozen=True)
+class RayMaterializationResult(MaterializationResult[ray.ObjectRef]):
+    _partition: ray.ObjectRef
+
+    def partition(self) -> ray.ObjectRef:
+        return self._partition
+
+    def metadata(self) -> PartitionMetadata:
+        return ray.get(get_meta.remote(self._partition))
+
+    def cancel(self) -> None:
+        return ray.cancel(self._partition)
+
+    def _noop(self, _: ray.ObjectRef) -> None:
+        return None
